#
import time
from machine import UART
#
import Define
import Lines as LIN
import Thread as THR
#
class CUart():
    #---------------------------------------------------------
    #   Constructor
    #---------------------------------------------------------
    def __init__(self, uartid, txpin, rxpin):
        self.Uart = UART(uartid, tx=txpin, rx=rxpin)
        self.TxLines = LIN.CLines()
        self.RxLines = LIN.CLines() 
        self.OnRxLine = None
        self.OnTxLine = None
        self.Thread = THR.CThread(self.CBOnStart, self.CBOnBusy, self.CBOnAbort, self.CBOnEnd)
        self.RxLine = ''
    #---------------------------------------------------------
    #   Property
    #---------------------------------------------------------
    def SetOnRxLine(self, onrxline):
        self.OnRxLine = onrxline
    def SetOnTxLine(self, ontxline):
        self.OnTxLine = ontxline
    #---------------------------------------------------------
    #   Callback Thread
    #---------------------------------------------------------
    def CBOnStart(self, thread):
        return
    def CBOnBusy(self, thread):
        while (THR.stBusy == thread.State):
            # first: RxData
            while (0 < self.Uart.any()):
                C = self.Uart.read(1)
                if (b'\r' == C) or (b'\n' == C):
                    if (0 < len(self.RxLine)):
                        self.RxLines.Push(self.RxLine)
                        self.RxLine = '';
                else:
                    self.RxLine += C.decode("utf-8")
            if (0 < self.RxLines.Count()):
                Line = self.RxLines.Pop()
                if (None != self.OnRxLine):
                    self.OnRxLine(Line)
            # second: TxData
            if (0 < self.TxLines.Count()):
                Line = self.TxLines.Pop()
                self.Uart.write(Line + '\r\n')
                if (None != self.OnTxLine):
                    self.OnTxLine(Line)
    def CBOnAbort(self, thread):
        return
    def CBOnEnd(self, thread):
        return
    #---------------------------------------------------------
    #   Management
    #---------------------------------------------------------
    def Open(self):
        self.Uart.init(baudrate=115200)#, bits=8, parity=None, stop=1, cts=-1, rts=-1)
        self.Thread.Start()   
        self.RxLine = ''
        return
    #
    def Close(self):
        self.Thread.Abort()
        self.Uart.deinit()
        return 
    #
    def TxLine(self, line):
        self.TxLines.Push(line)
    #
#
#-------------------------------------------------------------
#   Uart - Main - Check Library
#-------------------------------------------------------------
def CBOnRxLine(rxline):
    print('CBOnRxLine[{}]'.format(rxline))
#
def CBOnTxLine(txline):
    print('CBOnTxLine[{}]'.format(txline))
#
if ('__main__' == __name__):    
    print('*** CheckUart: begin')
    #
    Uart = CUart(Define.ID_UART1, Define.PIN_UART1_TX, Define.PIN_UART1_RX)
    Uart.SetOnRxLine(CBOnRxLine)
    Uart.SetOnTxLine(CBOnTxLine)
    Uart.Open()
    #
    Uart.TxLine('000asdf')
    Uart.TxLine('111asdf')
    time.sleep(2.0)
    #
    Uart.Close()
    #
    print('*** CheckUart: end')
#    